#!/usr/bin/perl

use DBI;
use DBD::mysql;
use vars qw ( $db );

############
# DB Vars - You MUST change these to match a valid account on your database

print << "EOF";

###############################################################
We will need the credentials for a database user that
is allowed to make changes to the snort database schema.  
By default the snort user that demarc uses does not have 
these privileges.

Please make sure that the user supplied below has 
administrative privileges for the database 
(ie ALTER, CREATE, DROP).


###############################################################

EOF

print "DB USER? [snort] > ";
chomp (my $db_user = <STDIN>);
($db_user) || ($db_user = "snort");
print "DB PASSWORD? > ";
chomp (my $db_password = <STDIN>);
print "DB HOST? [localhost] > ";
chomp (my $db_host  = <STDIN>);
($db_host) || ($db_host = "localhost");
print "DB NAME? [snort] > ";
chomp (my $db_name    = <STDIN>);
($db_name) || ($db_name = "snort");

print << "EOF";

User: $db_user
Password: $db_password
Host: $db_host
Name: $db_name

EOF
print "Is this correct?[y/N] >";

chomp($huh = <STDIN>);
if ($huh!~/y/i){
	die "OK, come back later when you're ready.\n";
}
&connect_to_db || &error("Invalid credentials, please try again.");

print <<"EOF";
Which previous version of DEMARC are you upgrading FROM? 

1) 1.04.XX
2) 1.05-RC1
3) None, get me out of here.

EOF
print "1-3 > ";

chomp (my $ug_from    = <STDIN>);
my $filename;

if ($ug_from=~/^1$/){
	$filename = "db_patch_queries";
	open (IN,$filename) || &error("Cant find \"$filename\" file in current directory!");
}
elsif($ug_from=~/^2$/){
	$filename = "db_patch_queries-1.05-RC1";
	open (IN,$filename) || &error("Cant find \"$filename\" file in current directory!");
}
elsif($ug_from=~/^3$/){
	die "OK, goodbye.\n";
}
else{
	die "Invalid selection, please run the script again\n";
}


print "About to run the queries located in the file \"$filename\" against database \"$db_name\" as user \"$db_user\" on host \"$db_host\"...\n";
print "Are you sure you'd like to proceed? [Ny] > ";
chomp( $input = <STDIN> );

die "OK ... Aborting ...\n" unless $input=~/^y$/i;

my $sql_query;
while (<IN>){
	$sql_query .= $_ if ($_!~/^#/);
	if ($_=~/\;/){
		$sql_query=~s/\;//g;
		$db_ptr = &run_query($sql_query);
		$sql_query = ();
	}
}

print "X"x50 . "\n";
print "Congratulations. The database has been patched.\n";
print "X"x50 . "\n";



#############################################
# Functions
#############################################
sub run_query{
my ($sql_query) = @_;
&connect_to_db();
$q_count++;
print "Running Query $q_count\n";

my $prepared = $db->prepare($sql_query);
if (!$prepared){
        #Then there's an error with the syntax of the query:
        &error("Syntax Error in SQL Query:\n$sql_query\n:\n".$db->errstr);
}
my $dbquery = $prepared->execute();
if (my $error_string = $prepared->errstr){
        &error("$error_string : $sql_query");
}
return $prepared;
}
#####################
sub connect_to_db{
if (!$db || !$db->ping){

# function args override defaults, but error out if neither are there.
 $db_user         ||  &error("No database user specified.");
 $db_password     ||  &error("No database password specified.");
 $db_host         ||  &error("No database host specified.");
 $db_name         ||  &error("No database specified.");

 $db = DBI->connect("dbi:mysql:host=$db_host;database=$db_name", $db_user , $db_password) || &error("Connection Problem! ". $DBI::errstr);

}
return $db;
}
####################
sub error{
my ($le) = @_;
die "$le\n";
}
##################
sub safe_slash{
my ($string_ref) = @_;

$$string_ref=~s/(['\\])/\\$1/g;

}
#################

